"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const Types_1 = require("frontblock-generic/Types");
const logger = Types_1.getLogger("frontblock-api-client", 'info');
const bsock = require('bsock');
/**
 * Frontblock api connection lib
 */
class FrontblockApiClient {
    constructor(conf) {
        this.conf = conf;
        this.started = false;
        this.apikey = "";
    }
    consume(apikey, uid, callback) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const r = yield this.socket.call("consume", apikey, uid);
            const res = Types_1.parseSubResponse(r);
            if (res instanceof Types_1.SubscriptionResponse) {
                this.socket.hook(res.uid, (tx) => {
                    const deserialized = JSON.parse(tx);
                    callback(deserialized);
                });
            }
            return res;
        });
    }
    quit(apikey, consumerUid) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const r = yield this.socket.call("quit", apikey, consumerUid);
            const res = Types_1.parseResponse(r);
            return res;
        });
    }
    subscribe(apikey, coin, account) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const r = yield this.socket.call("subscribe", apikey, coin, account);
            const res = Types_1.parseSubResponse(r);
            return res;
        });
    }
    subsume(apikey, coin, account, callback) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const r = yield this.socket.call("subsume", apikey, coin, account);
            const res = Types_1.parseSubResponse(r);
            if (res instanceof Types_1.SubscriptionResponse) {
                this.socket.hook(res.uid, (tx) => {
                    const deserialized = JSON.parse(tx);
                    callback(deserialized);
                });
            }
            return res;
        });
    }
    unsubscribe(apikey, uid) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const r = yield this.socket.call('unsubscribe', apikey, uid);
            const res = Types_1.parseResponse(r);
            if (res instanceof Types_1.SuccessResponse)
                this.socket.unhook(uid);
            return res;
        });
    }
    getPluginList() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield this.socket.call('getPluginList');
        });
    }
    connect() {
        if (this.started) {
            logger.warn("FrontblockApiClient has already been started. Ignoring");
            return;
        }
        logger.info("Starting apiClient with ", this.conf);
        this.socket = bsock.connect(this.conf.apiPort, this.conf.apiHost, this.conf.tls != null ? this.conf.tls : false);
        this.started = true;
    }
    disconnect() {
        if (!this.started) {
            logger.warn("FrontblockApiClient has not been started. Ignoring");
            return;
        }
        this.socket.close();
    }
}
exports.FrontblockApiClient = FrontblockApiClient;
